package com.supervision.handler.graph;

import cn.hutool.core.collection.CollUtil;
import com.supervision.exception.ItemNotFoundException;
import com.supervision.ngbatis.dao.CommonQueryDao;
import com.supervision.ngbatis.domain.tag.ItemBranch;
import com.supervision.ngbatis.domain.tag.ItemLeaf;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 查询图数据库,寻找符合条件的节点
 */
@Slf4j
@Component
public class FindItemNodeHandler {

    @Resource
    private CommonQueryDao commonQueryDao;

    public List<ItemLeaf> findAllMatchLeafNode(String itemName) {
        log.info("根据事项名称{}寻找可能匹配的节点", itemName);
        // 首先查询叶子节点的数据,如果有,就直接按照叶子节点的数据来匹配
        List<ItemLeaf> leafList = commonQueryDao.findItemLeafNodeVertexByItemName(itemName);
        if (CollUtil.isEmpty(leafList)) {
            List<ItemBranch> itemBranchNodeVertexByItemName = commonQueryDao.findItemBranchNodeVertexByItemName(itemName);
            // 如果分支节点不为空,那么就从分支节点去找叶子节点
            if (CollUtil.isNotEmpty(itemBranchNodeVertexByItemName)) {
                List<ItemLeaf> leafListTemp = new ArrayList<>();
                // 遍历分支节点,找到叶子节点
                for (ItemBranch itemBranch : itemBranchNodeVertexByItemName) {
                    List<ItemLeaf> itemLeafByBranchId = commonQueryDao.findItemLeafByBranchId(itemBranch.getVid());
                    if (CollUtil.isNotEmpty(itemLeafByBranchId)) {
                        leafListTemp.addAll(itemLeafByBranchId);
                    }
                }
                leafList.addAll(leafListTemp);
            }
        }
        if (CollUtil.isEmpty(leafList)) {
            throw new ItemNotFoundException("未找到与事项匹配的节点");
        }
        return leafList;
    }
}
